from datasette import Forbidden


async def iterate_searchable_tables(datasette, request):
    for db_name, database in datasette.databases.items():
        hidden_tables = set(await database.hidden_table_names())
        for table in await database.table_names():
            if table in hidden_tables:
                continue
            fts_table = await database.fts_table(table)
            if fts_table:
                # Check user has permission to view that table
                try:
                    await datasette.ensure_permissions(
                        request.actor,
                        [
                            ("view-table", (database.name, table)),
                            ("view-database", database.name),
                            "view-instance",
                        ],
                    )
                    yield (db_name, table)
                except Forbidden:
                    pass


async def has_searchable_tables(datasette, request):
    # Return True on the first table we find
    async for _ in iterate_searchable_tables(datasette, request):
        return True
    return False


async def get_searchable_tables(datasette, request):
    tables = []
    async for table in iterate_searchable_tables(datasette, request):
        tables.append(table)
    return tables



import os
from setuptools import setup

VERSION = "1.1.1"


def get_long_description():
    with open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md"),
        encoding="utf8",
    ) as fp:
        return fp.read()


setup(
    name="datasette-search-all",
    description="Datasette plugin for searching all searchable tables at once",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="Simon Willison",
    url="https://github.com/simonw/datasette-search-all",
    project_urls={
        "Issues": "https://github.com/simonw/datasette-search-all/issues",
        "CI": "https://github.com/simonw/datasette-search-all/actions",
        "Changelog": "https://github.com/simonw/datasette-search-all/releases",
    },
    license="Apache License, Version 2.0",
    version=VERSION,
    packages=["datasette_search_all"],
    entry_points={"datasette": ["search_all = datasette_search_all"]},
    package_data={"datasette_search_all": ["templates/*.html"]},
    install_requires=["datasette>=0.63.1"],
    python_requires=">=3.7",
    extras_require={"test": ["pytest", "pytest-asyncio", "sqlite-utils"]},
    tests_require=["datasette-search-all[test]"],
)



from datasette import hookimpl
from datasette.utils.asgi import Response
from .utils import get_searchable_tables, has_searchable_tables
import json


@hookimpl
def menu_links(datasette, request):
    async def inner():
        if not request:
            return None
        if await has_searchable_tables(datasette, request):
            return [
                {"href": datasette.urls.path("/-/search"), "label": "Search all tables"}
            ]

    return inner


async def search_all(datasette, request):
    searchable_tables = list(await get_searchable_tables(datasette, request))
    tables = [
        {
            "database": database,
            "table": table,
            "url": datasette.urls.table(database, table),
            "url_json": datasette.urls.table(database, table, format="json"),
        }
        for database, table in searchable_tables
    ]
    return Response.html(
        await datasette.render_template(
            "search_all.html",
            {
                "q": request.args.get("q") or "",
                "searchable_tables": tables,
                "searchable_tables_json": json.dumps(tables, indent=4),
            },
            request=request,
        )
    )


@hookimpl
def extra_template_vars(template, datasette, request):
    if template != "index.html":
        return
    # Add list of searchable tables
    async def inner():
        searchable_tables = list(await get_searchable_tables(datasette, request))
        return {"searchable_tables": searchable_tables}

    return inner


@hookimpl
def register_routes():
    return [
        ("/-/search", search_all),
    ]
